﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Interactivity;
using System.ComponentModel;
using System;
using System.Linq;
using System.Collections.ObjectModel;
using System.Windows.Input;
using System.Reflection;
using System.Collections.Generic;
using ViewModelTreeControl;

namespace Behaviors
{
    [System.ComponentModel.Description("Collapse Tree Nodes")]
    public class TreeCollapseBehavior : TargetedTriggerAction<TreeView>, INotifyPropertyChanged
    {
        private TreeView objTreeView;

        protected override void OnAttached()
        {
            base.OnAttached();

            objTreeView = (TreeView)(this.AssociatedObject);
        }

        protected override void Invoke(object parameter)
        {
            CollapseTree();
        }

        void objTreeView_Loaded(object sender, RoutedEventArgs e)
        {

        }

        #region CollapseTree
        private void CollapseTree()
        {
            // Refresh Tree Control
            objTreeView.UpdateLayout();
            // Get the DataContext of the Tree Control
            MainPageViewModel objMainPageViewModel = (MainPageViewModel)objTreeView.DataContext;

            // Get collection of items bound to Tree Control
            ObservableCollection<Category> colCategories = (ObservableCollection<Category>)objMainPageViewModel.colCategory;

            if (colCategories != null)
            {
                // Loop through the top levels items
                foreach (var Cat in colCategories)
                {
                    // Get Parent Node
                    var result = (from objCategory in Cat.AllChildren()
                                  select objCategory).FirstOrDefault();

                    // Get the Tree Control node container for the item
                    TreeViewItem objTreeViewItem = (TreeViewItem)objTreeView.ItemContainerGenerator.ContainerFromItem(Cat);

                    if (objTreeViewItem != null)
                    {
                        // Collapse the Node
                        objTreeViewItem.IsExpanded = false;

                        // Refresh Tree Control
                        objTreeView.UpdateLayout();

                        CollapseChildNode(objTreeViewItem, Cat);
                    }
                }
            }
        }

        // This method collapses child nodes
        private void CollapseChildNode(TreeViewItem objTreeViewItem, Category Cat)
        {
            // Loop through all the sub Categories
            foreach (var item in Cat.Categories)
            {
                // Find the Node
                var result = (from objCategory in item.AllChildren()
                              select objCategory).FirstOrDefault();

                // Get the Tree Control node container for the item
                TreeViewItem SubTreeViewItem = (TreeViewItem)objTreeViewItem.ItemContainerGenerator.ContainerFromItem(item);

                if (SubTreeViewItem != null)
                {
                    // Collapse the Node
                    SubTreeViewItem.IsExpanded = false;

                    // Refresh Tree Control
                    objTreeView.UpdateLayout();

                    // Recursively call the function                
                    CollapseChildNode(SubTreeViewItem, item);
                }
            }
        }
        #endregion

        // Utility

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }
        #endregion
    }
}
