﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Interactivity;
using System.ComponentModel;
using System;
using System.Linq;
using System.Collections.ObjectModel;
using System.Windows.Input;
using System.Reflection;
using System.Collections.Generic;
using ViewModelTreeControl;

namespace Behaviors
{
    [System.ComponentModel.Description("Expands Tree Node")]
    public class TreeExpandBehavior : TargetedTriggerAction<TreeView>, INotifyPropertyChanged
    {
        private TreeView objTreeView;

        protected override void OnAttached()
        {
            base.OnAttached();

            objTreeView = (TreeView)(this.AssociatedObject);
        }

        protected override void Invoke(object parameter)
        {
            SelectNode();
        }

        void objTreeView_Loaded(object sender, RoutedEventArgs e)
        {
            SelectNode();
        }

        #region SelectNode
        private void SelectNode()
        {
            // Refresh Tree Control
            objTreeView.UpdateLayout();
            // Get the DataContext of the Tree Control
            MainPageViewModel objMainPageViewModel = (MainPageViewModel)objTreeView.DataContext;

            // Get collection of items bound to Tree Control
            ObservableCollection<Category> colCategories = (ObservableCollection<Category>)objMainPageViewModel.colCategory;

            if (colCategories != null)
            {
                // Loop through the top levels items
                foreach (var Cat in colCategories)
                {
                    // Find the Node - Is it a child of this parent?
                    var result = (from objCategory in Cat.AllChildren()
                                  where objCategory.IsSelected == true
                                  select objCategory).FirstOrDefault();

                    // If the selected item is a child of the Parent
                    if (result != null)
                    {
                        // Get the Tree Control node container for the item
                        TreeViewItem objTreeViewItem = (TreeViewItem)objTreeView.ItemContainerGenerator.ContainerFromItem(Cat);
                        // Expand the Node
                        objTreeViewItem.IsExpanded = true;

                        // Refresh Tree Control
                        objTreeView.UpdateLayout();

                        ExpandChildNode(objTreeViewItem, Cat);
                    }
                }
            }
        }

        // This method expands child nodes
        private void ExpandChildNode(TreeViewItem objTreeViewItem, Category Cat)
        {
            // Loop through all the sub Categories
            foreach (var item in Cat.Categories)
            {
                // Find the Node - Is it a child of this parent?
                var result = (from objCategory in item.AllChildren()
                              where objCategory.IsSelected == true
                              select objCategory).FirstOrDefault();

                // If the selected item is a child of the Parent
                if (result != null)
                {
                    // Get the Tree Control node container for the item
                    TreeViewItem SubTreeViewItem = (TreeViewItem)objTreeViewItem.ItemContainerGenerator.ContainerFromItem(item);
                    // Expand the Node
                    SubTreeViewItem.IsExpanded = true;

                    // Refresh Tree Control
                    objTreeView.UpdateLayout();

                    // Recursively call the function                
                    ExpandChildNode(SubTreeViewItem, item);
                }
            }
        }
        #endregion

        // Utility

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }
        #endregion
    }
}
