﻿// Copyright (c) 2010
// by Michael Washington

using System;
using System.Linq;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Input;
using System.Windows;
using System.Collections.Generic;
using System.Windows.Data;
using System.Collections.Specialized;

namespace ViewModelTreeControl
{
    public class MainPageViewModel : INotifyPropertyChanged
    {

        public MainPageViewModel()
        {
            GetCategoriesCommand = new DelegateCommand(GetCategories, CanGetCategories);
            ClearCategoriesCommand = new DelegateCommand(ClearCategories, CanClearCategories);
            SetCategoryCommand = new DelegateCommand(SetCategory, CanSetCategory);

            // Load the Categories
            GetCategories(null);
        }

        // Commands

        #region GetCategoriesCommand
        public ICommand GetCategoriesCommand { get; set; }
        public void GetCategories(object param)
        {
            colCategory.Clear();

            Category objCategorySub0 = new Category();
            objCategorySub0.IsSelected = false;
            objCategorySub0.CategoryName = "Category Sub1-1";
            objCategorySub0.Categories = new ObservableCollection<Category>();

            Category objCategorySub01 = new Category();
            objCategorySub01.IsSelected = false;
            objCategorySub01.CategoryName = "Category Sub2-1";
            objCategorySub01.Categories = new ObservableCollection<Category>();

            Category objCategorySub1 = new Category();
            objCategorySub1.IsSelected = false;
            objCategorySub1.CategoryName = "Category One Sub1";
            objCategorySub1.Categories = new ObservableCollection<Category>();
            objCategorySub1.Categories.Add(objCategorySub0);

            Category objCategorySub2 = new Category();
            objCategorySub2.IsSelected = false;
            objCategorySub2.CategoryName = "Category One Sub2";
            objCategorySub2.Categories = new ObservableCollection<Category>();
            objCategorySub2.Categories.Add(objCategorySub01);

            Category objCategory1 = new Category();
            objCategory1.IsSelected = false;
            objCategory1.CategoryName = "Category One";
            objCategory1.Categories = new ObservableCollection<Category>();
            objCategory1.Categories.Add(objCategorySub1);
            objCategory1.Categories.Add(objCategorySub2);

            Category objCategorySub3 = new Category();
            objCategorySub3.IsSelected = false;
            objCategorySub3.CategoryName = "Category Two Sub1";
            objCategorySub3.Categories = new ObservableCollection<Category>();

            Category objCategorySub4 = new Category();
            objCategorySub4.IsSelected = false;
            objCategorySub4.CategoryName = "Category Two Sub2";
            objCategorySub4.Categories = new ObservableCollection<Category>();

            Category objCategory2 = new Category();
            objCategory2.IsSelected = false;
            objCategory2.CategoryName = "Category Two";
            objCategory2.Categories = new ObservableCollection<Category>();
            objCategory2.Categories.Add(objCategorySub3);
            objCategory2.Categories.Add(objCategorySub4);

            colCategory.Add(objCategory1);
            colCategory.Add(objCategory2);
        }

        private bool CanGetCategories(object param)
        {
            return true;
        }
        #endregion

        #region ClearCategoriesCommand
        public ICommand ClearCategoriesCommand { get; set; }
        public void ClearCategories(object param)
        {
            // Clear the selection off all Categories
            foreach (var Cat in colCategory)
            {
                // Set IsSelected to false for the Category
                Cat.IsSelected = false;

                // This Linq statement iterates through all children 
                // and sets their IsSelected to false
                // Because it is not performing an evaluation
                // it is setting a property 
                var result = (from objCategory in Cat.AllChildren()
                              where objCategory.IsSelected = false
                              select objCategory).ToList();
            }

            // Change the value of ExpandSelectedTreeNodes to False
            ExpandSelectedTreeNodes = false;

            // This forces anything bound to is, to be notified
            // That the value has been updated so that
            // Any Behavior bound to it is fired
            this.NotifyPropertyChanged("ExpandSelectedTreeNodes");
        }

        private bool CanClearCategories(object param)
        {
            return true;
        }
        #endregion

        #region SetCategoryCommand
        public ICommand SetCategoryCommand { get; set; }
        public void SetCategory(object param)
        {
            // Set Nodes as Selected
            foreach (var Cat in colCategory)
            {
                // Find the Node
                var result = from objCategory in Cat.AllChildren()
                              where 
                              (objCategory.CategoryName == "Category Sub2-1" ||
                               objCategory.CategoryName == "Category Two Sub2")
                              select objCategory;

                foreach (var item in result)
                {
                    // Set the IsSelected property so the checkbox
                    // will be checked
                    item.IsSelected = true;
                }

                // Change the value of ExpandSelectedTreeNodes to True
                ExpandSelectedTreeNodes = true;

                // This forces anything bound to is, to be notified
                // That the value has been updated so that
                // Any Behavior bound to it is fired
                this.NotifyPropertyChanged("ExpandSelectedTreeNodes");
            }
        }

        private bool CanSetCategory(object param)
        {
            return true;
        }
        #endregion

        // Properties

        #region ExpandSelectedTreeNodes
        private bool _ExpandSelectedTreeNodes = false;
        public bool ExpandSelectedTreeNodes
        {
            get { return _ExpandSelectedTreeNodes; }
            private set
            {
                if (ExpandSelectedTreeNodes == value)
                {
                    return;
                }
                _ExpandSelectedTreeNodes = value;
                this.NotifyPropertyChanged("ExpandSelectedTreeNodes");
            }
        }
        #endregion

        // Collections

        #region colCategory
        private ObservableCollection<Category> _colCategory
            = new ObservableCollection<Category>();
        public ObservableCollection<Category> colCategory
        {
            get { return _colCategory; }
            set
            {
                if (colCategory == value)
                {
                    return;
                }
                _colCategory = value;
                this.NotifyPropertyChanged("colCategory");
            }
        }
        #endregion

        // Utility

        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }
        #endregion
    }
}
